'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _elasticsearch = require('elasticsearch');

var _elasticsearch2 = _interopRequireDefault(_elasticsearch);

var _joi = require('joi');

var _joi2 = _interopRequireDefault(_joi);

var _errorCustom = require('@packt/error-custom');

var _errorCustom2 = _interopRequireDefault(_errorCustom);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/**
 * A simplified interface class for ElasticSearch
 */
class ElasticInterface {
  /**
   * Constructor for the ElasticInterface
   * @param {Object} [config]
   * Configuration Object
   *
   * @param {Object} [config.host]
   * URL for the ES host
   *
   * @param {Object} [config.indexName]
   * The name for the index used to store the log
   */
  constructor(config = {}) {
    const options = Object.assign({
      maxRetries: 1,
      requestTimeout: 500,
      deadTimeout: 500,
      pingTimeout: 500
    }, config);

    // Check a host is set
    if (!options.host && process.env.AUDIT_ES) {
      options.host = process.env.AUDIT_ES;
    }

    if (!options.host) {
      throw new _errorCustom2.default('No ElasticSearch host provided', 500, 1000400);
    }

    const date = new Date();
    this.elasticIndex = options.indexName || `${date.getUTCFullYear()}-${`0${date.getUTCDate()}`.slice(-2)}`;
    this.elasticClient = new _elasticsearch2.default.Client(options);
    this.defineJoiSchema();
  }

  /**
   * Assigns the Joi object to this.joiSchema for validation
   * @return {Void}
   */
  defineJoiSchema() {
    this.joiSchema = {
      userId: _joi2.default.string().guid().required(),
      queryType: _joi2.default.string().required(),
      query: _joi2.default.string().required()
    };
  }

  /**
   * Validated the auditObject before sending it to ES
   * @param  {Object} auditObject
   * An object of all the auditable information
   *
   * @return {Object}
   * An object containing Joi errors if the validation fails
   */
  validateAuditInformation(auditObject) {
    return _joi2.default.validate(auditObject, this.joiSchema);
  }

  /**
   * Find or Create the index used for audit logging
   *
   * @param  {String} indexName
   * Name for the index used to log against
   *
   * @return {Promise}
   * Promise containing the raw information for the index
   */
  findOrCreateIndex(indexName = '') {
    if (typeof indexName !== 'string') {
      throw new _errorCustom2.default('No indexName provided.', 500, 1000401);
    }

    return new Promise((resolve, reject) => {
      this.elasticClient.indices.exists({ index: indexName }, (error, body) => {
        if (error) {
          reject(error);
        }

        if (!body) {
          const newIndex = {
            index: indexName,
            body: {
              mappings: {}
            }
          };
          this.elasticClient.indices.create(newIndex, (errorCreate, bodyCreate) => {
            if (errorCreate) {
              reject(errorCreate);
            } else {
              resolve(bodyCreate);
            }
          });
        } else {
          const returnBody = {
            index: indexName
          };
          resolve(returnBody);
        }
      });
    });
  }

  /**
   * Send the log to the Elastic instance
   *
   * @param  {Object} logObject
   * The object being send to log
   *
   * @param  {Object} logObject.userId
   * UUID representing the user in querion
   *
   * @param  {Object} logObject.queryType
   * If the query is a CREATE|UPDATE|DELETE
   *
   * @param  {Object} logObject.query
   * The actual query executed
   *
   * @return {Promise}
   * Returned object for a successful send to ES
   */
  sendLog(logObject) {
    return new Promise((resolve, reject) => {
      const validation = this.validateAuditInformation(logObject);
      if (!validation.error) {
        this.findOrCreateIndex(this.elasticIndex).then(() => {
          const date = new Date();
          const completeLog = Object.assign(logObject, { created_at: date.toISOString() });
          this.elasticClient.index({
            index: this.elasticIndex,
            type: 'doc',
            body: completeLog
          }, (error, body) => {
            if (error) {
              reject(error);
            } else {
              resolve(body);
            }
          });
        });
      } else {
        reject(validation);
      }
    });
  }
}

exports.default = ElasticInterface;