'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _joi = require('joi');

var _joi2 = _interopRequireDefault(_joi);

var _errorCustom = require('@packt/error-custom');

var _errorCustom2 = _interopRequireDefault(_errorCustom);

var _elasticInterface = require('./elastic-interface');

var _elasticInterface2 = _interopRequireDefault(_elasticInterface);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/**
 * Helper to add the audit log directly into sequelize
 */
class SequelizeHelper {
  /**
   * Constructor for the SequelizeHelper
   *
   * @param {Object} [elasticInterface]
   * The elastic interface
   */
  constructor(elasticInterface) {
    if (!(elasticInterface instanceof _elasticInterface2.default)) {
      throw new _errorCustom2.default('Must be a instance of ElasticInterface', 500, 1000402);
    }

    this.elastic = elasticInterface;
  }

  /**
   * Set's the user ID
   *
   * @param {String} userId
   * The User ID to applied (must be a UUID)
   */
  setUserId(userId) {
    const validation = _joi2.default.validate(userId, _joi2.default.string().guid().required());
    if (validation.error) {
      throw new _errorCustom2.default('User ID must be a UUID', 500, 1000403);
    }

    this.userId = userId;

    return this.userId;
  }

  /**
   * Sequelize afterCreate hook
   *
   * @param  {Sequelize} instance
   * Instance of Sequelize used
   *
   * @return {Void}
   */
  afterCreate(instance) {
    this.elastic.sendLog({
      userId: this.userId,
      queryType: 'CREATE',
      query: JSON.stringify(instance)
    });
  }

  /**
   * Sequelize afterDestroy hook
   *
   * @param  {Sequelize} instance
   * Instance of Sequelize used
   *
   * @return {Void}
   */
  afterDestroy(instance) {
    this.elastic.sendLog({
      userId: this.userId,
      queryType: 'DELETE',
      query: JSON.stringify(instance)
    });
  }

  /**
   * Sequelize afterUpdate hook
   *
   * @param  {Sequelize} instance
   * Instance of Sequelize used
   *
   * @return {Void}
   */
  afterUpdate(instance) {
    this.elastic.sendLog({
      userId: this.userId,
      queryType: 'UPDATE',
      query: JSON.stringify(instance)
    });
  }

  /**
   * Sequelize afterSave hook
   *
   * @param  {Sequelize} instance
   * Instance of Sequelize used
   *
   * @return {Void}
   */
  afterSave(instance) {
    this.elastic.sendLog({
      userId: this.userId,
      queryType: 'CREATE',
      query: JSON.stringify(instance)
    });
  }

  /**
   * Sequelize afterUpsert hook
   *
   * @param  {Sequelize} instance
   * Instance of Sequelize used
   *
   * @return {Void}
   */
  afterUpsert(instance) {
    this.elastic.sendLog({
      userId: this.userId,
      queryType: 'UPSERT',
      query: JSON.stringify(instance)
    });
  }
}

exports.default = SequelizeHelper;