import ElasticSearch from 'elasticsearch';
import Joi from 'joi';
import ErrorCustom from '@packt/error-custom';

/**
 * A simplified interface class for ElasticSearch
 */
class ElasticInterface {
  /**
   * Constructor for the ElasticInterface
   * @param {Object} [config]
   * Configuration Object
   *
   * @param {Object} [config.host]
   * URL for the ES host
   *
   * @param {Object} [config.indexName]
   * The name for the index used to store the log
   */
  constructor(config = {}) {
    const options = Object.assign({
      maxRetries: 1,
      requestTimeout: 500,
      deadTimeout: 500,
      pingTimeout: 500,
    }, config);

    // Check a host is set
    if (!options.host && process.env.AUDIT_ES) {
      options.host = process.env.AUDIT_ES;
    }

    if (!options.host) {
      throw new ErrorCustom('No ElasticSearch host provided', 500, 1000400);
    }

    const date = new Date();
    this.elasticIndex = options.indexName || `${date.getUTCFullYear()}-${(`0${date.getUTCDate()}`).slice(-2)}`;
    this.elasticClient = new ElasticSearch.Client(options);
    this.defineJoiSchema();
  }

  /**
   * Assigns the Joi object to this.joiSchema for validation
   * @return {Void}
   */
  defineJoiSchema() {
    this.joiSchema = {
      userId: Joi.string().guid().required(),
      queryType: Joi.string().required(),
      query: Joi.string().required(),
    };
  }

  /**
   * Validated the auditObject before sending it to ES
   * @param  {Object} auditObject
   * An object of all the auditable information
   *
   * @return {Object}
   * An object containing Joi errors if the validation fails
   */
  validateAuditInformation(auditObject) {
    return Joi.validate(auditObject, this.joiSchema);
  }

  /**
   * Find or Create the index used for audit logging
   *
   * @param  {String} indexName
   * Name for the index used to log against
   *
   * @return {Promise}
   * Promise containing the raw information for the index
   */
  findOrCreateIndex(indexName = '') {
    if (typeof indexName !== 'string') {
      throw new ErrorCustom('No indexName provided.', 500, 1000401);
    }

    return new Promise((resolve, reject) => {
      this.elasticClient.indices.exists({ index: indexName }, (error, body) => {
        if (error) {
          reject(error);
        }

        if (!body) {
          const newIndex = {
            index: indexName,
            body: {
              mappings: {},
            },
          };
          this.elasticClient.indices.create(newIndex, (errorCreate, bodyCreate) => {
            if (errorCreate) {
              reject(errorCreate);
            } else {
              resolve(bodyCreate);
            }
          });
        } else {
          const returnBody = {
            index: indexName,
          };
          resolve(returnBody);
        }
      });
    });
  }

  /**
   * Send the log to the Elastic instance
   *
   * @param  {Object} logObject
   * The object being send to log
   *
   * @param  {Object} logObject.userId
   * UUID representing the user in querion
   *
   * @param  {Object} logObject.queryType
   * If the query is a CREATE|UPDATE|DELETE
   *
   * @param  {Object} logObject.query
   * The actual query executed
   *
   * @return {Promise}
   * Returned object for a successful send to ES
   */
  sendLog(logObject) {
    return new Promise((resolve, reject) => {
      const validation = this.validateAuditInformation(logObject);
      if (!validation.error) {
        this.findOrCreateIndex(this.elasticIndex)
          .then(() => {
            const date = new Date();
            const completeLog = Object.assign(logObject, { created_at: date.toISOString() });
            this.elasticClient.index({
              index: this.elasticIndex,
              type: 'doc',
              body: completeLog,
            }, (error, body) => {
              if (error) {
                reject(error);
              } else {
                resolve(body);
              }
            });
          });
      } else {
        reject(validation);
      }
    });
  }
}

export default ElasticInterface;
