import Joi from 'joi';
import ErrorCustom from '@packt/error-custom';
import ElasticInterface from './elastic-interface';


/**
 * Helper to add the audit log directly into sequelize
 */
class SequelizeHelper {
  /**
   * Constructor for the SequelizeHelper
   *
   * @param {Object} [elasticInterface]
   * The elastic interface
   */
  constructor(elasticInterface) {
    if (!(elasticInterface instanceof ElasticInterface)) {
      throw new ErrorCustom('Must be a instance of ElasticInterface', 500, 1000402);
    }

    this.elastic = elasticInterface;
  }

  /**
   * Set's the user ID
   *
   * @param {String} userId
   * The User ID to applied (must be a UUID)
   */
  setUserId(userId) {
    const validation = Joi.validate(userId, Joi.string().guid().required());
    if (validation.error) {
      throw new ErrorCustom('User ID must be a UUID', 500, 1000403);
    }

    this.userId = userId;

    return this.userId;
  }

  /**
   * Sequelize afterCreate hook
   *
   * @param  {Sequelize} instance
   * Instance of Sequelize used
   *
   * @return {Void}
   */
  afterCreate(instance) {
    this.elastic.sendLog({
      userId: this.userId,
      queryType: 'CREATE',
      query: JSON.stringify(instance),
    });
  }

  /**
   * Sequelize afterDestroy hook
   *
   * @param  {Sequelize} instance
   * Instance of Sequelize used
   *
   * @return {Void}
   */
  afterDestroy(instance) {
    this.elastic.sendLog({
      userId: this.userId,
      queryType: 'DELETE',
      query: JSON.stringify(instance),
    });
  }

  /**
   * Sequelize afterUpdate hook
   *
   * @param  {Sequelize} instance
   * Instance of Sequelize used
   *
   * @return {Void}
   */
  afterUpdate(instance) {
    this.elastic.sendLog({
      userId: this.userId,
      queryType: 'UPDATE',
      query: JSON.stringify(instance),
    });
  }

  /**
   * Sequelize afterSave hook
   *
   * @param  {Sequelize} instance
   * Instance of Sequelize used
   *
   * @return {Void}
   */
  afterSave(instance) {
    this.elastic.sendLog({
      userId: this.userId,
      queryType: 'CREATE',
      query: JSON.stringify(instance),
    });
  }

  /**
   * Sequelize afterUpsert hook
   *
   * @param  {Sequelize} instance
   * Instance of Sequelize used
   *
   * @return {Void}
   */
  afterUpsert(instance) {
    this.elastic.sendLog({
      userId: this.userId,
      queryType: 'UPSERT',
      query: JSON.stringify(instance),
    });
  }
}

export default SequelizeHelper;
