import { expect } from 'chai'; // eslint-disable-line
import sinon from 'sinon';
import UUID from 'uuid/v4';
import ErrorCustom from '@packt/error-custom';
import ElasticInterface from '../src/elastic-interface';

const elasticHost = 'http://localhost:9200';

describe('Elastic Interface', () => {
  /* Constructor */
  describe('Constructing the instance', () => {
    it('Create an instance with a host set', (done) => {
      const elasticInterface = new ElasticInterface({
        host: elasticHost,
      });
      const date = new Date();
      expect(elasticInterface.elasticIndex).to.be.eql(`${date.getUTCFullYear()}-${(`0${date.getUTCDate()}`).slice(-2)}`);
      expect(elasticInterface.joiSchema).to.be.instanceof(Object);
      done();
    });

    it('Create an instance with a host & indexName set', (done) => {
      const elasticInterface = new ElasticInterface({
        host: elasticHost,
        indexName: 'test-index',
      });
      expect(elasticInterface.elasticIndex).to.be.eql('test-index');
      expect(elasticInterface.joiSchema).to.be.instanceof(Object);
      done();
    });

    it('Create an instance with no host and no ENV', (done) => {
      try {
        const elasticInterface = new ElasticInterface();
        expect(0).to.equal('happy path fail');
      } catch (e) {
        expect(e).to.be.instanceof(ErrorCustom);
        expect(e.message).to.be.eql('No ElasticSearch host provided');
        done();
      }
    });

    it('Create an instance with no host and ENV', (done) => {
      process.env.AUDIT_ES = elasticHost;
      const elasticInterface = new ElasticInterface();
      const date = new Date();
      expect(elasticInterface.elasticIndex).to.be.eql(`${date.getUTCFullYear()}-${(`0${date.getUTCDate()}`).slice(-2)}`);
      expect(elasticInterface.joiSchema).to.be.instanceof(Object);
      delete process.env.AUDIT_ES;
      done();
    });
  });

  /* Check the validation */
  describe('Validation of Joi', () => {
    it('Valid audit object', (done) => {
      const elasticInterface = new ElasticInterface({
        host: elasticHost,
      });
      const validate = elasticInterface.validateAuditInformation({
        userId: UUID(),
        queryType: 'CREATE',
        query: 'SELECT * FROM users',
      });
      expect(validate.error).to.be.eql(null);
      done();
    });

    it('Invalid audit object - userId', (done) => {
      const elasticInterface = new ElasticInterface({
        host: elasticHost,
      });
      const validate = elasticInterface.validateAuditInformation({
        queryType: 'CREATE',
        query: 'SELECT * FROM users',
      });
      expect(validate.error.message).to.be.eql('child "userId" fails because ["userId" is required]');
      done();
    });

    it('Invalid audit object - queryType', (done) => {
      const elasticInterface = new ElasticInterface({
        host: elasticHost,
      });
      const validate = elasticInterface.validateAuditInformation({
        userId: UUID(),
        query: 'SELECT * FROM users',
      });
      expect(validate.error.message).to.be.eql('child "queryType" fails because ["queryType" is required]');
      done();
    });

    it('Invalid audit object - query', (done) => {
      const elasticInterface = new ElasticInterface({
        host: elasticHost,
      });
      const validate = elasticInterface.validateAuditInformation({
        userId: UUID(),
        queryType: 'CREATE',
      });
      expect(validate.error.message).to.be.eql('child "query" fails because ["query" is required]');
      done();
    });
  });

  /* Manage the index */
  describe('Elastic Index', () => {
    it('Working index creation', (done) => {
      const elasticInterface = new ElasticInterface({
        host: elasticHost,
      });
      const indexName = UUID();
      elasticInterface.findOrCreateIndex(indexName)
        .then((data) => {
          expect(data.index).to.be.eql(indexName);
          done();
        });
    });

    it('Index already exists', (done) => {
      const elasticInterface = new ElasticInterface({
        host: elasticHost,
      });
      const indexName = UUID();
      elasticInterface.findOrCreateIndex(indexName)
        .then(() => elasticInterface.findOrCreateIndex(indexName))
        .then((data) => {
          expect(data.index).to.be.eql(indexName);
          done();
        });
    });

    it('Incorrect indexName type', async () => {
      const elasticInterface = new ElasticInterface({
        host: elasticHost,
      });
      try {
        await elasticInterface.findOrCreateIndex(null);
        expect(0).to.equal('happy path fail');
      } catch (error) {
        elasticInterface.elasticClient.close();
        expect(error).to.be.instanceOf(Error);
        expect(error.message).to.be.eql('No indexName provided.');
      }
    }).timeout(10000);

    it('Bad elastic host', async () => {
      const elasticInterface = new ElasticInterface({
        host: 'http://localhost:65535',
      });
      const indexName = UUID();
      try {
        await elasticInterface.findOrCreateIndex(indexName);
        expect(0).to.equal('happy path fail');
      } catch (error) {
        elasticInterface.elasticClient.close();
        expect(error).to.be.instanceOf(Error);
        expect(error.message).to.be.eql('No Living connections');
      }
    }).timeout(10000);
  });

  /* Manage the index */
  describe('Send audit log', () => {
    it('Correct log', (done) => {
      const elasticInterface = new ElasticInterface({
        host: elasticHost,
        indexName: UUID(),
      });
      elasticInterface.sendLog({
        userId: UUID(),
        queryType: 'CREATE',
        query: 'INSET INTO (id) VALUES (1);',
      })
        .then((data) => {
          expect(data.result).to.be.eql('created');
          done();
        });
    });

    it('Invalid log object - userId', (done) => {
      const elasticInterface = new ElasticInterface({
        host: elasticHost,
      });
      elasticInterface.sendLog({
        queryType: 'CREATE',
        query: 'INSERT INTO (id) VALUES (1);',
      })
        .then(() => {
          expect(0).to.equal('happy path fail');
        })
        .catch((error) => {
          expect(error.error.message).to.be.eql('child "userId" fails because ["userId" is required]');
          done();
        });
    });

    it('Invalid log object - queryType', (done) => {
      const elasticInterface = new ElasticInterface({
        host: elasticHost,
      });
      elasticInterface.sendLog({
        userId: UUID(),
        query: 'INSERT INTO (id) VALUES (1);',
      })
        .then(() => {
          expect(0).to.equal('happy path fail');
        })
        .catch((error) => {
          expect(error.error.message).to.be.eql('child "queryType" fails because ["queryType" is required]');
          done();
        });
    });

    it('Invalid log object - query', (done) => {
      const elasticInterface = new ElasticInterface({
        host: elasticHost,
      });
      elasticInterface.sendLog({
        userId: UUID(),
        queryType: 'CREATE',
      })
        .then(() => {
          expect(0).to.equal('happy path fail');
        })
        .catch((error) => {
          expect(error.error.message).to.be.eql('child "query" fails because ["query" is required]');
          done();
        });
    });

    it('Error logging', async () => {
      const elasticInterface = new ElasticInterface({
        host: elasticHost,
      });
      sinon.stub(elasticInterface.elasticClient, 'index').callsFake((data, callback) => callback(new Error('broken')));
      try {
        await elasticInterface.sendLog({
          userId: UUID(),
          queryType: 'CREATE',
          query: 'INSERT INTO (id) VALUES (1);',
        });
        expect(0).to.equal('happy path fail');
      } catch (error) {
        expect(error.message).to.equal('broken');
      }
    });
  });
});
