import { expect } from 'chai'; // eslint-disable-line
import uuid from 'uuid/v4';
import ErrorCustom from '@packt/error-custom';
import Sequelize from 'sequelize';
import sinon from 'sinon';
import SequelizeHelper from '../src/sequelize-helper';
import ElasticInterface from '../src/elastic-interface';

const elasticHost = 'http://localhost:9200';
const postgres = {
  dbName: process.env.DB_NAME,
  dbUser: process.env.DB_USER,
  dbPass: process.env.DB_PASSWORD,
  dbHost: process.env.DB_HOST,
};
const sequelizeModel = {
  id: {
    type: Sequelize.UUID,
    allowNull: false,
    primaryKey: true,
    defaultValue: Sequelize.UUIDV4,
  },
  sqlCheck: Sequelize.STRING,
  createdAt: {
    allowNull: false,
    field: 'created_at',
    type: Sequelize.DATE,
  },
  updatedAt: {
    allowNull: false,
    field: 'updated_at',
    type: Sequelize.DATE,
  },
};

describe('SequelizeHelper Interface', () => {
  before((done) => {
    const sequelize = new Sequelize(postgres.dbName, postgres.dbUser, postgres.dbPass, {
      host: postgres.dbHost,
      dialect: 'postgres',
    });
    sequelize.queryInterface.createTable(
      'tests',
      {
        id: {
          type: Sequelize.UUID,
          primaryKey: true,
          defaultValue: Sequelize.UUIDV4,
        },
        sqlCheck: Sequelize.STRING,
        createdAt: {
          field: 'created_at',
          type: Sequelize.DATE,
        },
        updatedAt: {
          field: 'updated_at',
          type: Sequelize.DATE,
        },
      },
    ).then((data) => {
      sequelize.close();
      done();
    });
  });

  /* Constructor */
  describe('Constructing the instance', () => {
    it('Create instance with a host', (done) => {
      const elasticInterface = new ElasticInterface({
        host: elasticHost,
        indexName: uuid(),
      });
      const sequelizeHelper = new SequelizeHelper(elasticInterface);
      expect(sequelizeHelper.elastic).to.be.instanceof(ElasticInterface);
      done();
    });

    it('Create instance without ElasticInterface', (done) => {
      try {
        const sequelizeHelper = new SequelizeHelper();
        expect(0).to.equal('happy path fail');
      } catch (e) {
        expect(e).to.be.instanceof(ErrorCustom);
        expect(e.message).to.be.eql('Must be a instance of ElasticInterface');
        done();
      }
    });
  });

  /* Constructor */
  describe('Set User ID', () => {
    it('Correct User ID', (done) => {
      const elasticInterface = new ElasticInterface({
        host: elasticHost,
        indexName: uuid(),
      });
      const userId = uuid();
      const sequelizeHelper = new SequelizeHelper(elasticInterface);
      sequelizeHelper.setUserId(userId);
      expect(sequelizeHelper.userId).to.be.eql(userId);
      done();
    });

    it('Invalid User ID', (done) => {
      const elasticInterface = new ElasticInterface({
        host: elasticHost,
        indexName: uuid(),
      });
      const userId = `${uuid()}-blah-broken-boo`;
      const sequelizeHelper = new SequelizeHelper(elasticInterface);
      try {
        sequelizeHelper.setUserId(userId);
        expect(0).to.equal('happy path fail');
      } catch (e) {
        expect(e).to.be.instanceOf(ErrorCustom);
        expect(e.message).to.be.eql('User ID must be a UUID');
        done();
      }
    });
  });

  /* Sequelize Hook Helpers */
  describe('Sequelize Hooks', () => {
    it('afterCreate', async () => {
      const elasticInterface = new ElasticInterface({
        host: elasticHost,
        indexName: uuid(),
      });
      const sequelizeHelper = new SequelizeHelper(elasticInterface);
      const userId = uuid();
      sequelizeHelper.setUserId(userId);
      const spy = sinon.stub(sequelizeHelper.elastic, 'sendLog');
      spy.returns(Promise.resolve(true));

      const sequelize = new Sequelize(postgres.dbName, postgres.dbUser, postgres.dbPass, {
        host: postgres.dbHost,
        dialect: 'postgres',
        define: {
          hooks: {
            afterCreate: (instance, options) => {
              sequelizeHelper.afterCreate(instance, options);
            },
          },
        },
      });
      const Tests = sequelize.define('tests', sequelizeModel);
      await Tests.create({
        sqlCheck: 'CREATE',
      });

      sequelize.close();
      expect(sequelizeHelper.elastic).to.be.instanceof(ElasticInterface);
      expect(spy.called).to.equal(true);
      expect(spy.args[0][0].userId).to.equal(userId);
      expect(spy.args[0][0].queryType).to.equal('CREATE');
    });

    it('afterDestroy', async () => {
      const elasticInterface = new ElasticInterface({
        host: elasticHost,
        indexName: uuid(),
      });
      const sequelizeHelper = new SequelizeHelper(elasticInterface);
      const userId = uuid();
      sequelizeHelper.setUserId(userId);
      const spy = sinon.stub(sequelizeHelper.elastic, 'sendLog');
      spy.returns(Promise.resolve(true));

      const sequelize = new Sequelize(postgres.dbName, postgres.dbUser, postgres.dbPass, {
        host: postgres.dbHost,
        dialect: 'postgres',
        define: {
          hooks: {
            afterDestroy: (instance, options) => {
              sequelizeHelper.afterDestroy(instance, options);
            },
            beforeBulkDestroy: (options) => {
              options.individualHooks = true;
              return options;
            },
          },
        },
      });
      const Tests = sequelize.define('tests', sequelizeModel);
      await Tests.create({
        sqlCheck: 'CREATE',
      })
        .then(data => Tests.destroy({
          where: { id: data.dataValues.id },
        }));
      sequelize.close();

      expect(sequelizeHelper.elastic).to.be.instanceof(ElasticInterface);
      expect(spy.called).to.equal(true);
      expect(spy.args[0][0].userId).to.equal(userId);
      expect(spy.args[0][0].queryType).to.equal('DELETE');
    });

    it('afterUpdate', async () => {
      const elasticInterface = new ElasticInterface({
        host: elasticHost,
        indexName: uuid(),
      });
      const sequelizeHelper = new SequelizeHelper(elasticInterface);
      const userId = uuid();
      sequelizeHelper.setUserId(userId);
      const spy = sinon.stub(sequelizeHelper.elastic, 'sendLog');
      spy.returns(Promise.resolve(true));

      const sequelize = new Sequelize(postgres.dbName, postgres.dbUser, postgres.dbPass, {
        host: postgres.dbHost,
        dialect: 'postgres',
        define: {
          hooks: {
            afterUpdate: (instance, options) => {
              sequelizeHelper.afterUpdate(instance, options);
            },
            beforeBulkUpdate: (options) => {
              options.individualHooks = true;
              return options;
            },
          },
        },
      });
      const Tests = sequelize.define('tests', sequelizeModel);
      await Tests.create({
        sqlCheck: 'CREATE',
      })
        .then(data => Tests.update({
          sqlCheck: 'UPDATE',
        }, {
          where: { id: data.id },
        }));
      sequelize.close();

      expect(sequelizeHelper.elastic).to.be.instanceof(ElasticInterface);
      expect(spy.called).to.equal(true);
      expect(spy.args[0][0].userId).to.equal(userId);
      expect(spy.args[0][0].queryType).to.equal('UPDATE');
    });

    it('afterSave', async () => {
      const elasticInterface = new ElasticInterface({
        host: elasticHost,
        indexName: uuid(),
      });
      const sequelizeHelper = new SequelizeHelper(elasticInterface);
      const userId = uuid();
      sequelizeHelper.setUserId(userId);
      const spy = sinon.stub(sequelizeHelper.elastic, 'sendLog');
      spy.returns(Promise.resolve(true));

      const sequelize = new Sequelize(postgres.dbName, postgres.dbUser, postgres.dbPass, {
        host: postgres.dbHost,
        dialect: 'postgres',
        define: {
          hooks: {
            afterSave: (instance, options) => {
              sequelizeHelper.afterSave(instance, options);
            },
          },
        },
      });
      const Tests = sequelize.define('tests', sequelizeModel);
      const testCase = Tests.build({
        sqlCheck: 'CREATE',
      });
      await testCase.save();
      sequelize.close();

      expect(sequelizeHelper.elastic).to.be.instanceof(ElasticInterface);
      expect(spy.called).to.equal(true);
      expect(spy.args[0][0].userId).to.equal(userId);
      expect(spy.args[0][0].queryType).to.equal('CREATE');
    });

    it('afterUpsert', async () => {
      const elasticInterface = new ElasticInterface({
        host: elasticHost,
        indexName: uuid(),
      });
      const sequelizeHelper = new SequelizeHelper(elasticInterface);
      const userId = uuid();
      sequelizeHelper.setUserId(userId);
      const spy = sinon.stub(sequelizeHelper.elastic, 'sendLog');
      spy.returns(Promise.resolve(true));

      const sequelize = new Sequelize(postgres.dbName, postgres.dbUser, postgres.dbPass, {
        host: postgres.dbHost,
        dialect: 'postgres',
        define: {
          hooks: {
            afterUpsert: (instance, options) => {
              sequelizeHelper.afterUpsert(instance, options);
            },
          },
        },
      });
      const Tests = sequelize.define('tests', sequelizeModel);
      await Tests.upsert({
        sqlCheck: 'CREATE',
      });

      sequelize.close();
      expect(sequelizeHelper.elastic).to.be.instanceof(ElasticInterface);
      expect(spy.called).to.equal(true);
      expect(spy.args[0][0].userId).to.equal(userId);
      expect(spy.args[0][0].queryType).to.equal('UPSERT');
    });
  });
});
